@@ -1,4 +1,4 @@
-// Copyright (c) 1997  Per M.A. Bothner.
+// Copyright (c) 1997, 2004  Per M.A. Bothner.
 // This is free software;  for terms and warranty disclaimer see ./COPYING.
 
 package gnu.bytecode;
@@ -61,6 +61,16 @@ public class LocalVarsAttr extends Attribute
       }
   }
 
+  /** Mark scopes upto specified scope as 'preserved'.
+   * This means that the local variable slots are not available for reuse
+   * when the scope exits.  The effected scopes are all ancestor scopes from
+   * the current scope (inclusive) upto the specified scope (exclusive). */
+  public void preserveVariablesUpto(Scope scope)
+  {
+    for (Scope cur = current_scope;  cur != scope;  cur = cur.parent)
+      cur.preserved = true;
+  }
+
   public final boolean isEmpty ()
   {
     VarEnumerator vars = allVars();
@@ -80,7 +90,7 @@ public class LocalVarsAttr extends Attribute
     Variable var;
     while ((var = vars.nextVar ()) != null)
       {
-	if (var.isSimple () && var.name != null)
+	if (var.shouldEmit())
 	  local_variable_count++;
       }
     return local_variable_count;
@@ -118,13 +128,16 @@ public class LocalVarsAttr extends Attribute
 	    
     for (vars.reset (); (var = vars.nextVar ()) != null; )
       {
-	if (var.isSimple () && var.name != null)
+	if (var.shouldEmit())
 	  {
-	    dstr.writeShort (var.start_pc);
-	    dstr.writeShort (var.end_pc - var.start_pc);
-	    dstr.writeShort (var.name_index);
-	    dstr.writeShort (var.signature_index);
-	    dstr.writeShort (var.offset);
+	    Scope scope = var.scope;
+	    int start_pc = scope.start.position;
+	    int end_pc = scope.end.position;
+	    dstr.writeShort(start_pc);
+	    dstr.writeShort(end_pc - start_pc);
+	    dstr.writeShort(var.name_index);
+	    dstr.writeShort(var.signature_index);
+	    dstr.writeShort(var.offset);
 	  }
       }
   }
@@ -153,9 +166,18 @@ public class LocalVarsAttr extends Attribute
 	    dst.printOptionalIndex(var.signature_index);
 	    dst.printSignature(var.getType());
 	    dst.print(" (pc: ");
-	    dst.print (var.start_pc);
-	    dst.print(" length: ");
-	    dst.print (var.end_pc - var.start_pc);
+	    Scope scope = var.scope;
+	    int start_pc, end_pc;
+	    if (scope == null || scope.start == null || scope.end == null
+		|| (start_pc = scope.start.position) < 0
+		|| (end_pc = scope.end.position) < 0)
+	      dst.print("unknown");
+	    else
+	      {
+		dst.print(start_pc);
+		dst.print(" length: ");
+		dst.print(end_pc - start_pc);
+	      }
 	    dst.println(')');
 	  }
       }
