@@ -1,4 +1,4 @@
-// Copyright (c) 2001  Per M.A. Bothner.
+// Copyright (c) 2001, 2004, 2005  Per M.A. Bothner.
 // This is free software;  for terms and warranty disclaimer see ./COPYING.
 
 package gnu.commonlisp.lang;
@@ -10,6 +10,8 @@ import kawa.standard.Scheme;
 import gnu.bytecode.Type;
 import gnu.kawa.lispexpr.LangPrimType;
 import gnu.kawa.functions.DisplayFormat;
+import gnu.kawa.functions.NumberCompare;
+import gnu.kawa.lispexpr.ReadTable;
 
 public class CommonLisp extends Lisp2
 {
@@ -45,67 +47,70 @@ public class CommonLisp extends Lisp2
     return (char) i;
   }
 
-  public gnu.text.Lexer getLexer(InPort inp, gnu.text.SourceMessages messages)
-  {
-    return new CLispReader(inp, messages);
-  }
-
   public String getName()
   {
     return "CommonLisp";
   }
 
-  static CommonLisp instance;
+  // This field need to be public so that the findLiteral method in
+  // gnu.expr.LitTable can find it.
+  public static final CommonLisp instance;
 
-  protected void defun(String name, Object value)
+  public static final Environment clispEnvironment
+    = Environment.make("clisp-environment");
+
+  public static final NumberCompare numEqu;
+  public static final NumberCompare numGrt;
+  public static final NumberCompare numGEq;
+  public static final NumberCompare numLss;
+  public static final NumberCompare numLEq;
+
+  static
   {
-    Symbols.setFunctionBinding(environ, name, value);
-    if (value instanceof Named)
+    instance = new CommonLisp();
+
+    instance.define("t", TRUE);
+    instance.define("nil", FALSE);
+    numEqu = NumberCompare.make(instance, "=",
+                                NumberCompare.TRUE_IF_EQU);
+    numGrt = NumberCompare.make(instance, ">",
+                                NumberCompare.TRUE_IF_GRT);
+    numGEq = NumberCompare.make(instance, ">=",
+                                NumberCompare.TRUE_IF_GRT|NumberCompare.TRUE_IF_EQU);
+    numLss = NumberCompare.make(instance, "<",
+                                NumberCompare.TRUE_IF_LSS);
+    numLEq = NumberCompare.make(instance, "<=",
+                                NumberCompare.TRUE_IF_LSS|NumberCompare.TRUE_IF_EQU);
+    CallContext ctx = CallContext.getInstance();
+    Environment saveEnv = ctx.getEnvironmentRaw();
+    try
       {
-	Named n = (Named) value;
-	if (n.getName() == null)
-	  n.setName(name);
+        ctx.setEnvironmentRaw(clispEnvironment);
+        instance.initLisp();
+      }
+    finally
+      {
+        ctx.setEnvironmentRaw(saveEnv);
       }
   }
 
-  private void defun(Procedure proc)
-  {
-    defun(proc.getName(), proc);
-  }
-
-  static int lispCounter = 0;
-
   public CommonLisp()
   {
-    Environment scmEnv = Scheme.builtin();
-    environ = SymbolTable.make("interaction-environment."+(++lispCounter));
-    Environment.setCurrent(environ);
+    environ = clispEnvironment;
+  }
 
-    TRUE = environ.getSymbol("t");
-    TRUE.set(TRUE);
-    define("nil", FALSE);
-
-    SymbolEnumeration e = scmEnv.enumerateAllSymbols();
+  void initLisp()
+  {
+    LocationEnumeration e = Scheme.builtin().enumerateAllLocations();
     while (e.hasMoreElements())
       {
-	Symbol b = e.nextSymbol();
-	Object val = b.get(null);
-	if (val != null)
-	  {
-	    String name = b.getName();
-	    if (val instanceof Procedure || val instanceof kawa.lang.Syntax)
-	      defun(name, val);
-	    else
-	      define(name, val);
-	  }
+        importLocation(e.nextLocation());
       }
 
-    if (instance == null)
-      instance = this;
-
     try
       {
 	// Force it to be loaded now, so we can over-ride let* length etc.
+	loadClass("kawa.lib.prim_syntax");
 	loadClass("kawa.lib.std_syntax");
 	loadClass("kawa.lib.lists");
 	loadClass("kawa.lib.strings");
@@ -117,9 +122,9 @@ public class CommonLisp extends Lisp2
       }
 
     kawa.lang.Lambda lambda = new kawa.lang.Lambda();
-    lambda.setKeywords(getSymbol("&optional"),
-		       getSymbol("&rest"),
-		       getSymbol("&key"));
+    lambda.setKeywords(asSymbol("&optional"),
+		       asSymbol("&rest"),
+		       asSymbol("&key"));
     lambda.defaultDefault = nilExpr;
     defun("lambda", lambda);
     defun("defun", new defun(lambda));
@@ -132,9 +137,6 @@ public class CommonLisp extends Lisp2
     defun("prog1", new prog1("prog1", 1));
     defun("prog2", prog1.prog2);
     defun("progn", new kawa.standard.begin());
-    defun("if", new kawa.standard.ifp());
-    defun("or", new kawa.standard.and_or(false, this));
-    defun("and", new kawa.standard.and_or(true, this));
     defun("unwind-protect", new gnu.commonlisp.lang.UnwindProtect());
     Procedure not = new kawa.standard.not(this);
     defun("not", not);
@@ -144,43 +146,31 @@ public class CommonLisp extends Lisp2
     defun("typep", new gnu.kawa.reflect.InstanceOf(this));
     defun("princ", displayFormat);
     defun("prin1", writeFormat);
+
+    defProcStFld("=", "gnu.commonlisp.lang.CommonLisp", "numEqu");
+    defProcStFld("<", "gnu.commonlisp.lang.CommonLisp", "numLss");
+    defProcStFld(">", "gnu.commonlisp.lang.CommonLisp", "numGrt");
+    defProcStFld("<=", "gnu.commonlisp.lang.CommonLisp", "numLEq");
+    defProcStFld(">=", "gnu.commonlisp.lang.CommonLisp", "numGEq");
+
+    defProcStFld("functionp", "gnu.commonlisp.lisp.PrimOps");
   }
 
   public static CommonLisp getInstance()
   {
-    if (instance == null)
-      {
-        Environment saveEnv = Environment.getCurrent();
-        try
-          {
-            instance = new CommonLisp();
-          }
-        finally
-          {
-            Environment.setCurrent(saveEnv);
-          }
-      }
     return instance;
   }
 
   /** The compiler insert calls to this method for applications and applets. */
   public static void registerEnvironment()
   {
-    CommonLisp interp = new CommonLisp();
-    Interpreter.defaultInterpreter = interp;
-    Environment.setCurrent(interp.getEnvironment());
+    Language.setDefaults(instance);
   }
 
-  public Object read (InPort in)
-    throws java.io.IOException, gnu.text.SyntaxException
-  {
-    return CLispReader.readObject(in);
-  }
+  static final AbstractFormat writeFormat = new DisplayFormat(true, 'C');
+  static final AbstractFormat displayFormat = new DisplayFormat(false, 'C');
 
-  static final DisplayFormat writeFormat = new DisplayFormat(true, 'C');
-  static final DisplayFormat displayFormat = new DisplayFormat(false, 'C');
-
-  public FormatToConsumer getFormat(boolean readable)
+  public AbstractFormat getFormat(boolean readable)
   {
     return readable ? writeFormat : displayFormat;
   }
@@ -209,23 +199,4 @@ public class CommonLisp extends Lisp2
       }
     return Type.make(clas);
   }
-
-  /** Import all the public fields of an object. */
-  public void defineFromFieldValue(String name, Object part)
-    throws Throwable
-  {
-    if (part instanceof Named)
-      name = ((Named) part).getName();
-    else
-      name = name.intern();
-    if (part instanceof Symbol)
-      environ.addSymbol((Symbol) part);
-    else if (part instanceof Procedure
-	     || part instanceof kawa.lang.Syntax)
-      Symbols.setFunctionBinding(environ, name, part);
-    else
-      environ.define(name, part);
-  }
 }
-
-
