@@ -1,6 +1,6 @@
 ;;; For now these are written in Scheme.
 ;;; They should be re-written in Common Lisp, but there are still some
-;;; limitations in the Common Lisp supprt making that difficult.
+;;; limitations in the Common Lisp support making that difficult.
 
 (define (car x)
   (if (eq? x '()) x (slot-ref (as <pair> x) 'car)))
@@ -16,16 +16,16 @@
 
 ;; SYMBOLS
 
-(define (boundp symbol) :: <clisp:boolean>
-  ((primitive-static-method <gnu.commonlisp.lang.Symbol> "isBound"
+(define (boundp symbol) :: |<clisp:boolean>|
+  ((primitive-static-method <gnu.commonlisp.lang.Symbols> "isBound"
                             <boolean> (<object>))
    symbol))
 
-(define (symbolp x) :: <clisp:boolean>
-  (invoke-static <gnu.commonlisp.lang.Symbol> 'isSymbol x))
+(define (symbolp x) :: |<clisp:boolean>|
+  (invoke-static <gnu.commonlisp.lang.Symbols> 'isSymbol x))
 
 (define (symbol-name symbol)
-  (invoke-static <gnu.commonlisp.lang.Symbol> 'getPrintName symbol))
+  (invoke-static <gnu.commonlisp.lang.Symbols> 'getPrintName symbol))
 
 ;(define (make-symbol NAME) ...)
 
@@ -37,33 +37,33 @@
 ;; mapatoms
 ;; unintern
 
-;;(define (symbol-plist symbol)
-;;  (invoke-static <gnu.commonlisp.lang.Symbol> 'getPropertyList symbol))
+(define (symbol-plist symbol)
+ (gnu.mapping.PropertyLocation:getPropertyList symbol))
 
-;;(define (setplist symbol plist)
-;;  (invoke-static <gnu.commonlisp.lang.Symbol> 'setPropertyList symbol plist)
-;;  plist)
+(define (setplist symbol plist)
+  (gnu.mapping.PropertyLocation:setPropertyList symbol plist)
+  plist)
 
 (define (plist-get plist prop #!optional default)
-  (invoke-static <gnu.commonlisp.lang.Symbol> 'plistGet plist prop default))
+  (gnu.mapping.PropertyLocation:plistGet plist prop default))
 
 (define (plist-put plist prop value)
-  (invoke-static <gnu.commonlisp.lang.Symbol> 'plistPut plist prop value))
+  (gnu.mapping.PropertyLocation:plistPut plist prop value))
 
 (define (plist-remprop plist prop)
-  (invoke-static <gnu.commonlisp.lang.Symbol> 'plistRemove plist prop))
+  (gnu.mapping.PropertyLocation:plistRemove plist prop))
 
 (define (plist-member plist prop)
   (if (eq?
-       (invoke-static <gnu.commonlisp.lang.Symbol> 'plistGet plist prop #!void)
+       (gnu.mapping.PropertyLocation:plistGet plist prop #!void)
        #!void)
       '() 't))
 
-(define (get (symbol :: <gnu.mapping.Binding>) property #!optional (default '()))
-  (invoke symbol 'getProperty property default))
+(define (get (symbol :: <gnu.mapping.Symbol>) property #!optional (default '()))
+  (gnu.mapping.PropertyLocation:getProperty symbol property default))
 
-(define (put (symbol :: <gnu.mapping.Binding>) property value)
-  (invoke symbol 'setProperty property value))
+(define (put symbol property value)
+  (gnu.mapping.PropertyLocation:putProperty symbol property value))
 
 ;; VARIABLES
 
@@ -71,14 +71,16 @@
 ;;     binding nor a global value.
 
 (define (symbol-value sym)
-  (invoke (invoke-static <gnu.commonlisp.lang.Symbol> 'getBinding sym) 'get))
+  (invoke (gnu.mapping.Environment:getCurrent) 'get (invoke-static <gnu.commonlisp.lang.Symbols> 'getSymbol sym)))
 
 ;; setq
 
 ;(define (make-symbol NAME)  ...)
 
 (define (set symbol value)
-  (invoke-static <gnu.commonlisp.lang.Symbol> 'setValueBinding symbol value))
+  ((gnu.mapping.Environment:getCurrent):put
+   (<gnu.commonlisp.lang.Symbols>:getSymbol symbol)
+   value))
 
 #|
 (define (add-to-list symbol value)
@@ -93,7 +95,7 @@
 ;; symbol's function cell is void, a `void-function' error is signaled.
 
 (define (symbol-function symbol)
-  (invoke-static <gnu.commonlisp.lang.Symbol> 'getFunctionBinding
+  (invoke-static <gnu.commonlisp.lang.Symbols> 'getFunctionBinding
 		 symbol))
 
 ;(define (fboundp x) ..)
@@ -101,24 +103,16 @@
 ;(fmakunboud symbol)
 
 (define (fset symbol object)
-  (invoke-static <gnu.commonlisp.lang.Symbol> 'setFunctionBinding
+  (invoke-static <gnu.commonlisp.lang.Symbols> 'setFunctionBinding
 		 (invoke-static <gnu.mapping.Environment> 'getCurrent)
 		 symbol object))
 
 (define (apply func #!rest (args :: <Object[]>))
-  (invoke-static 'gnu.kawa.functions.Apply 'applyN
-		 (if (symbol? func) (symbol-function func) func)
+  (invoke-static <gnu.kawa.functions.Apply> 'doApply
+		 (as <function>
+		     (if (symbol? func) (symbol-function func) func))
 		 args))
 
-(define-syntax prog1
-  (syntax-rules ()
-		((prog1 first)
-		 first)
-		((prog1 first rest ...)
-		 (let ((%prog1-save% first)) ;; Should be lexical-let? FIXME
-		   (begin rest ...)
-		   %prog1-save%))))
-
 ;;; ARRAYS
 
 (define (length (x :: <gnu.lists.Sequence>))
@@ -161,3 +155,6 @@
 
 (define (char-to-string ch)
   (make <string> 1 (invoke-static <gnu.commonlisp.lang.CommonLisp> 'asChar ch)))
+
+(define (functionp x) |<clisp:boolean>|
+  (instance? x <function>))
