@@ -1,11 +1,19 @@
+// Copyright (c) 2003  Per M.A. Bothner.
+// This is free software;  for terms and warranty disclaimer see ./COPYING.
+
 package gnu.expr;
 import java.util.Hashtable;
+import java.io.Externalizable;
+import gnu.bytecode.Type;
+import gnu.mapping.*;
 
 public class FindCapturedVars extends ExpWalker
 {
-  public static void findCapturedVars (Expression exp)
+  public static void findCapturedVars (Expression exp, Compilation comp)
   {
-    exp.walk(new FindCapturedVars());
+    FindCapturedVars walker = new FindCapturedVars();
+    walker.setContext(comp);
+    exp.walk(walker);
   }
 
   protected Expression walkApplyExp (ApplyExp exp)
@@ -20,17 +28,19 @@ public class FindCapturedVars extends ExpWalker
     // possible that we later find out that func needs a static link,
     // in which case the current function does as well;  this is taken
     // care of by calling setCallersNeedStaticLink in LambdaExp.)
-    if (exp.func instanceof ReferenceExp && ! Compilation.usingTailCalls)
+    if (exp.func instanceof ReferenceExp
+	&& Compilation.defaultCallConvention <= Compilation.CALL_WITH_RETURN)
       {
 	Declaration decl
 	  = Declaration.followAliases(((ReferenceExp) exp.func).binding);
-	if (decl != null && decl.context instanceof ModuleExp)
+	if (decl != null && decl.context instanceof ModuleExp
+            && ! decl.isPublic()
+            && ! decl.getFlag(Declaration.NONSTATIC_SPECIFIED))
 	  {
 	    Expression value = decl.getValue();
 	    if (value instanceof LambdaExp)
 	      {
 		LambdaExp lexp = (LambdaExp) value;
-		LambdaExp cur = getCurrentLambda();
 		if (! lexp.getNeedsClosureEnv())
 		  skipFunc = true;
 	      }
@@ -67,6 +77,16 @@ public class FindCapturedVars extends ExpWalker
       }
   }
 
+  protected Expression walkClassExp (ClassExp exp)
+  {
+    Expression ret = super.walkClassExp(exp);
+    // Make sure <init> has been declared, in case we need to invoke it.
+    // However, this is only safe to do if ! getNeedsClosureEnv().  FIXME.
+    if (! exp.explicitInit && ! exp.getNeedsClosureEnv())
+      Compilation.getConstructor(exp.instanceType, exp);
+    return ret;
+  }
+
   protected Expression walkModuleExp (ModuleExp exp)
   {
     ModuleExp saveModule = currentModule;
@@ -79,37 +99,6 @@ public class FindCapturedVars extends ExpWalker
       }
     finally
       {
-	if (unknownDecls != null)
-	  {
-	    int count = unknownDecls.size();
-	    java.util.Enumeration e = unknownDecls.keys();
-	    int i = 0;
-	    Expression[] init = new Expression[1];
-	    LetExp let = new LetExp(init);
-	    Declaration env =
-	      let.addDeclaration("env$",
-				 Compilation.typeEnvironment);
-	    init[0] = new ApplyExp(Compilation.getCurrentEnvironmentMethod,
-				   Expression.noExpressions);
-	    env.setCanRead(true);
-	    env.noteValue(init[0]);
-	    Expression[] exps = new Expression[count+1];
-	    for (;  e.hasMoreElements();  i++)
-	      {
-		String id = (String) e.nextElement();
-		Declaration decl = (Declaration) unknownDecls.get(id);
-		Expression[] args = new Expression[2];
-		args[0] = new ReferenceExp(env);
-		args[1] = new QuoteExp(id);
-		SetExp set = new SetExp(decl, 
-					new ApplyExp(Compilation.getBindingEnvironmentMethod, args));
-		set.setDefining(true);
-		exps[i] = set;
-	      }
-	    exps[i] = currentModule.body;
-	    let.setBody(new BeginExp(exps));
-	    currentModule.body = let;
-	  }
 	currentModule = saveModule;
 	unknownDecls = saveDecls;
       }
@@ -119,9 +108,12 @@ public class FindCapturedVars extends ExpWalker
   {
     for (Declaration decl = exp.firstDecl(); decl != null; decl = decl.nextDecl())
       {
-	Declaration bind = allocUnboundDecl(decl.getName());
-	capture(bind);
-	decl.base = bind;
+        if (decl.base == null)
+          {
+            Declaration bind = allocUnboundDecl(decl.getSymbol(), false);
+            capture(bind);
+            decl.base = bind;
+          }
       }
     return super.walkLetExp(exp);
   }
@@ -134,27 +126,37 @@ public class FindCapturedVars extends ExpWalker
 	// If init[i] is the magic QuoteExp.nullExp, and the real value
 	// is a LambdaExp or a QuoteExp, we're not going to get weird
 	// order-dependencies, and it is safe to transform it to a regular let.
+	// It's also necessary in the case of a LambdaExp if it shares
+	// a field with the declaration (see LambdaExp.allocFieldField),
+	// since assigning the nullExp can clobber the field after it has
+	// been initialized with a ModuleMethod.
 	Expression[] inits = exp.inits;
 	int len = inits.length;
-        BeginExp bexp = (BeginExp) exp.body;
-	Expression[] exps = bexp.exps;
-	if (bexp.length > len)
+	Expression[] exps = ((BeginExp) exp.body).exps;
+	int init_index = 0;
+	Declaration decl = exp.firstDecl();
+	for (int begin_index = 0;
+	     begin_index < exps.length && init_index < len;
+	     begin_index++)
 	  {
-	    int i = 0;
-	    Declaration decl = exp.firstDecl();
-	    for (; i < len; decl = decl.nextDecl(), i++)
+	    Expression st = exps[begin_index];
+	    if (st instanceof SetExp)
 	      {
-		if (inits[i] == QuoteExp.nullExp
-		    && exps[i] instanceof SetExp)
+		SetExp set = (SetExp) st;
+		if (set.binding == decl
+		    && inits[init_index] == QuoteExp.nullExp
+		    && set.isDefining())
 		  {
-		    SetExp set = (SetExp) exps[i];
-		    if ((set.new_value instanceof LambdaExp
-			 || set.new_value instanceof QuoteExp)
-			&& set.binding == decl)
+		    Expression new_value = set.new_value;
+		    if ((new_value instanceof QuoteExp
+			 || new_value instanceof LambdaExp)
+			&& decl.getValue() == new_value)
 		      {
-			inits[i] = set.new_value;
-			exps[i] = QuoteExp.voidExp;
+			inits[init_index] = new_value;
+			exps[begin_index] = QuoteExp.voidExp;
 		      }
+		    init_index++;
+		    decl = decl.nextDecl();
 		  }
 	      }
 	  }
@@ -166,14 +168,8 @@ public class FindCapturedVars extends ExpWalker
   {
     if (! (decl.getCanRead() || decl.getCanCall()))
       return;
-
-    if (decl.getFlag(Declaration.IS_UNKNOWN))
-      return; // FIXME - for now, as long as unknows are static
     if (decl.field != null && decl.field.getStaticFlag())
       return;
-    if (decl.getFlag(Declaration.IS_CONSTANT)
-	&& decl.getValue() instanceof QuoteExp)
-      return;
 
     LambdaExp curLambda = getCurrentLambda ();
     LambdaExp declLambda = decl.getContext().currentLambda ();
@@ -213,7 +209,7 @@ public class FindCapturedVars extends ExpWalker
         curLambda = curReturn.context;
         chain = chain.nextSibling;
       }
-    if (Compilation.usingCPStyle())
+    if (comp.usingCPStyle())
       {
 	if (curLambda instanceof ModuleExp)
 	  return;
@@ -237,69 +233,89 @@ public class FindCapturedVars extends ExpWalker
 	else if (declValue == curLambda && ! decl.getCanRead())
           return;
       }
-    if (decl.getFlag(Declaration.STATIC_SPECIFIED))
-      decl.setSimple(false);
+
+    if (decl.getFlag(Declaration.IS_UNKNOWN))
+      {
+	// Don't create a closure for a static function/class.
+	for (LambdaExp parent = curLambda; ; parent = parent.outerLambda())
+	  {
+	    if (parent == declLambda)
+	      break;
+	    if (parent.nameDecl != null
+		&& parent.nameDecl.getFlag(Declaration.STATIC_SPECIFIED))
+	      {
+		decl.setFlag(Declaration.STATIC_SPECIFIED);
+		break;
+	      }
+	  }
+      }
+    if (decl.base != null)
+      {
+	decl.base.setCanRead(true);
+	capture(decl.base);
+      }
     else if (decl.getCanRead() || declValue == null)
       {
-	LambdaExp heapLambda = curLambda;
-	heapLambda.setImportsLexVars();
-	LambdaExp parent = heapLambda.outerLambda();
-	for (LambdaExp outer = parent;  outer != declLambda && outer != null; )
+	if (! decl.isStatic())
 	  {
-	    heapLambda = outer;
-	    if (! decl.getCanRead() && declValue == outer)
-	      break;
-	    heapLambda.setNeedsStaticLink();
-	    outer = heapLambda.outerLambda();
-	  }
-	if (decl.base != null)
-	  {
-	    decl.base.setCanRead(true);
-	    capture(decl.base);
-	  }
-	if (decl.isSimple())
-	  {	
-	    if (declLambda.capturedVars == null
-		&& ! (declLambda instanceof ModuleExp
-		      || declLambda instanceof ClassExp))
+	    LambdaExp heapLambda = curLambda;
+	    heapLambda.setImportsLexVars();
+	    LambdaExp parent = heapLambda.outerLambda();
+	    for (LambdaExp outer = parent;  outer != declLambda && outer != null; )
 	      {
-		declLambda.heapFrame = new gnu.bytecode.Variable("heapFrame");
-		declLambda.heapFrame.setArtificial(true);
-	      }
-	    decl.setSimple(false);
-	    if (! decl.isPublic())
-	      {
-		decl.nextCapturedVar = declLambda.capturedVars;
-		declLambda.capturedVars = decl;
+		heapLambda = outer;
+		if (! decl.getCanRead() && declValue == outer)
+		  break;
+		Declaration heapDecl = heapLambda.nameDecl;
+		if (heapDecl != null
+		    && heapDecl.getFlag(Declaration.STATIC_SPECIFIED))
+		  {
+		    comp.error('e', "static " + heapLambda.getName()
+			       + " references non-static " + decl.getName());
+		  }
+		heapLambda.setNeedsStaticLink();
+		outer = heapLambda.outerLambda();
 	      }
 	  }
+
+        declLambda.capture(decl);
       }
   }
 
   Hashtable unknownDecls = null;
   ModuleExp currentModule = null;
 
-  Declaration allocUnboundDecl(String name)
+  Declaration allocUnboundDecl(Object name, boolean function)
   {
     Declaration decl;
+    Object key = name;
+    if (function && name instanceof Symbol)
+      {
+	if (! getCompilation().getLanguage().hasSeparateFunctionNamespace())
+	  function = false;
+	else // FIXME maybe just use gnu.lists.Pair and remove KeyPair class?
+	  key = new KeyPair((Symbol) name, EnvironmentKey.FUNCTION);
+      }
     if (unknownDecls == null)
       {
 	unknownDecls = new Hashtable(100);
 	decl = null;
       }
     else
-      decl = (Declaration) unknownDecls.get(name);
+      decl = (Declaration) unknownDecls.get(key);
     if (decl == null)
       {
 	decl = currentModule.addDeclaration(name);
 	decl.setSimple(false);
 	decl.setPrivate(true);
+	if (function)
+	  decl.setProcedureDecl(true);
 	if (currentModule.isStatic())
 	  decl.setFlag(Declaration.STATIC_SPECIFIED);
 	decl.setCanRead(true);
 	decl.setFlag(Declaration.IS_UNKNOWN);
 	decl.setIndirectBinding(true);
-	unknownDecls.put(name, decl);
+	unknownDecls.put(key, decl);
       }
     return decl;
   }
@@ -309,19 +325,59 @@ public class FindCapturedVars extends ExpWalker
     Declaration decl = exp.getBinding();
     if (decl == null)
       {
-	decl = allocUnboundDecl(exp.getName());
+	decl = allocUnboundDecl(exp.getSymbol(),
+				exp.isProcedureName());
 	exp.setBinding(decl);
       }
-    else // FIXME remove else when IS_UNKNOWN decls are non-static
-    capture(Declaration.followAliases(decl));
+    if (decl.getFlag(Declaration.IS_UNKNOWN))
+      {
+	Type type = getCompilation().getLanguage().getTypeFor(exp);
+	if (type instanceof Externalizable && ! exp.getDontDereference())
+	  return new QuoteExp(type);
+
+	if (comp.getBooleanOption("warn-undefined-variable", false))
+	  {
+	    Object resolved
+	      = comp.resolve(exp.getSymbol(), exp.isProcedureName());
+	    if (resolved == null)
+	      comp.error('w', "no declaration seen for "+exp.getName(), exp);
+	  }
+      }
+
+    capture(exp.contextDecl(), decl);
     return exp;
   }
 
+  void capture (Declaration containing, Declaration decl)
+  {
+    if (decl.isAlias() && decl.value instanceof ReferenceExp)
+      {
+	ReferenceExp rexp = (ReferenceExp) decl.value;
+	Declaration orig = rexp.binding;
+	if (orig != null
+	    && (containing == null || ! orig.needsContext()))
+	  {
+	    capture(rexp.contextDecl(), orig);
+	    return;
+	  }
+      }
+    if (containing != null && decl.needsContext())
+      capture(containing);
+    else
+      capture(decl);
+  }
+
   protected Expression walkThisExp (ThisExp exp)
   {
-    // FIXME - not really right, but works in simple cases.
-    getCurrentLambda ().setImportsLexVars();
-    return exp;
+    if (exp.isForContext())
+      {
+        // This is an extension used by define_syntax.
+        // FIXME - not really right, but works in simple cases.
+        getCurrentLambda ().setImportsLexVars();
+        return exp;
+      }
+    else
+      return walkReferenceExp (exp);
   }
 
   protected Expression walkSetExp (SetExp exp)
@@ -329,11 +385,15 @@ public class FindCapturedVars extends ExpWalker
     Declaration decl = exp.binding;
     if (decl == null)
       {
-	decl = allocUnboundDecl(exp.getName());
+	decl = allocUnboundDecl(exp.getSymbol(), exp.isFuncDef());
 	exp.binding = decl;
       }
-    else // FIXME remove else when IS_UNKNOWN decls are non-static
-    capture(Declaration.followAliases(decl));
+    if (! decl.ignorable())
+      {
+	if (! exp.isDefining())
+	  decl = Declaration.followAliases(decl);
+	capture(exp.contextDecl(), decl);
+      }
     return super.walkSetExp(exp);
   }
 
