@@ -55,11 +55,7 @@ public class ModuleExp extends LambdaExp
       {
 	ArrayClassLoader loader = new ArrayClassLoader ();
         if (url == null)
-          {
-            CallContext ctx = CallContext.getInstance();
-            String base = ctx.getBaseUri();
-            url = URI_utils.toURL(base);
-          }
+          url = Path.currentPath().toURL();
         loader.setResourceContext(url);
 	comp.loader = loader;
 
@@ -228,6 +224,12 @@ public class ModuleExp extends LambdaExp
                   {
                     inst = clas.newInstance();
                   }
+                /* #ifdef use:java.lang.Throwable.getCause */
+                catch (ExceptionInInitializerError ex)
+                  {
+                    throw ex.getCause();
+                  }
+                /* #endif */
 
 		// Import declarations defined in module into the Environment.
 		for (Declaration decl = mexp.firstDecl();
