@@ -11,27 +11,32 @@ import gnu.expr.*;
 public class ConstantFunction0 extends Procedure0 implements CanInline
 {
   final Object value;
+  final QuoteExp constant;
 
   public ConstantFunction0(String name, Object value)
   {
     super(name);
     this.value = value;
+    this.constant = QuoteExp.getInstance(value);
+  }
+
+  public ConstantFunction0(String name, QuoteExp constant)
+  {
+    super(name);
+    this.value = constant.getValue();
+    this.constant = constant;
   }
 
   public Object apply0() { return value; }
 
-  public Expression inline (ApplyExp exp /*, SourceMessages messages*/)
+  public Expression inline (ApplyExp exp, ExpWalker walker)
   {
     int nargs = exp.getArgCount();
-    if (nargs != 0)
+    if (nargs != 0 && walker != null)
       {
-	/*
-	String message = WrongArguments(this, nargs);
-	ExrrorExp exp = new ErrorExp(message);
-	messages.error('w', filename, lne, column, message);
-	*/
-	return exp;
+	String message = WrongArguments.checkArgCount(this, nargs);
+	return walker.noteError(message);
       }
-    return new QuoteExp(value);
+    return constant;
   }
 }
