@@ -149,7 +149,7 @@ public abstract class AbstractSequence
    * @param descend if true do depth-first traversal.
    * @return poistion of next match or 0 if none found
    */
-  public int nextMatching(int startPos, ElementPredicate type,
+  public int nextMatching(int startPos, ItemPredicate type,
 			  int endPos, boolean descend)
   {
     if (descend)
@@ -534,7 +534,7 @@ public abstract class AbstractSequence
     return 0;
   }
 
-  public int firstChildPos (int ipos, ElementPredicate predicate)
+  public int firstChildPos (int ipos, ItemPredicate predicate)
   {
     int child = firstChildPos(ipos);
     if (child == 0)
@@ -808,10 +808,10 @@ public abstract class AbstractSequence
   {
     boolean isSequence = this instanceof Sequence;
     if (isSequence)
-      out.beginGroup("#sequence");
+      out.startElement("#sequence");
     consumePosRange(startPos(), endPos(), out);
     if (isSequence)
-      out.endGroup();
+      out.endElement();
   }
 
   public void toString (String sep, StringBuffer sbuf)
