@@ -18,11 +18,17 @@ public class GapVector extends AbstractSequence implements Sequence
   public GapVector(SimpleVector base)
   {
     this.base = base;
+    this.gapStart = 0;
+    this.gapEnd = base.size;
+  }
+
+  protected GapVector ()
+  {
   }
 
   public int size()
   {
-    return base.getBufferLength() - (gapEnd - gapStart);
+    return base.size - (gapEnd - gapStart);
   }
 
   public boolean hasNext(int ipos)
@@ -30,7 +36,7 @@ public class GapVector extends AbstractSequence implements Sequence
     int index = ipos >>> 1;
     if (index >= gapStart)
       index += gapEnd - gapStart;
-    return index < base.getBufferLength();
+    return index < base.size;
   }
 
   public int getNextKind(int ipos)
@@ -56,7 +62,7 @@ public class GapVector extends AbstractSequence implements Sequence
 
   public void fill (Object value)
   {
-    base.fill(gapEnd, base.getBufferLength(), value);
+    base.fill(gapEnd, base.size, value);
     base.fill(0, gapStart, value);
   }
 
@@ -87,14 +93,14 @@ public class GapVector extends AbstractSequence implements Sequence
   {
     if (size > gapEnd - gapStart)
       {
-	int oldLength = base.getBufferLength();
+	int oldLength = base.size;
         int newLength = oldLength < 16 ? 16 : 2 * oldLength;
 	int minLength = oldLength - (gapEnd - gapStart) + size;
 	if (newLength < minLength)
 	  newLength = minLength;
 	// FIXME  this does unneeded copying.
 	// It may also leave unwanted junk in the gap (gap for gc).
-	base.setBufferLength(newLength);
+	base.setSize(newLength);
 	int newGapEnd = newLength - oldLength + gapEnd;
 	base.shift(gapEnd, newGapEnd, oldLength - gapEnd);
 	gapEnd = newGapEnd;
@@ -109,6 +115,39 @@ public class GapVector extends AbstractSequence implements Sequence
       shiftGap(where);
   }
 
+  /** If needed, move the gap so the given segment is contiguous.
+   * @return the offset in the base array containing the segment,
+   * or -1 if the parameters are bad.
+   */
+   
+  public int getSegment (int where, int len)
+  {
+    int length = size();
+    if (where < 0 || where > length)
+      return -1;
+    if (len < 0)
+      len = 0;
+    else if (where + len > length)
+      len = length - where;
+    // if (len < 0 || where + len > length)
+    //   return -1;
+    if (where + len <= gapStart)
+      return where;
+    if (where >= gapStart)
+      return where + (gapEnd - gapStart);
+    // Shift the gap depending in which direction needs least copying.
+    if (gapStart - where > (len >> 1))
+      {
+	shiftGap(where + len);
+	return where;
+      }
+    else
+      {
+	shiftGap(where);
+	return where + (gapEnd - gapStart);
+      }
+  }
+
   protected int addPos (int ipos, Object value)
   {
     int index = ipos >>> 1;
@@ -130,7 +169,7 @@ public class GapVector extends AbstractSequence implements Sequence
     ipos0 >>>= 1;
     ipos1 >>>= 1;
     if (ipos0 > gapEnd)
-      shiftGap(ipos0);
+      shiftGap(ipos0-gapEnd+gapStart);
     else if (ipos1 < gapStart)
       shiftGap(ipos1);
     if (ipos0 < gapStart)
@@ -169,7 +208,7 @@ public class GapVector extends AbstractSequence implements Sequence
 
   protected int nextIndex(int ipos)
   {
-    int index = ipos == -1 ? base.getBufferLength() : ipos >>> 1;
+    int index = ipos == -1 ? base.size : ipos >>> 1;
     if (index > gapStart)
       index -= gapEnd - gapStart;
     return index;
