@@ -208,7 +208,7 @@ class MPN
   }
 
     /** Divide divident[0:len-1] by (unsigned int)divisor.
-     * Write result into quotient[0:len-1.
+     * Write result into quotient[0:len-1].
      * Return the one-word (unsigned) remainder.
      * OK for quotient==dividend.
      */
@@ -444,9 +444,9 @@ class MPN
     return size;
   }
 
-  /** Compare x[0:size-1] with y[0:size-1], treating them as unsigned integers.
-   * @result -1, 0, or 1 depending on if x<y, x==y, or x>y.
-   * This is basically the same as gmp's mpn_cmp function.
+  /** Compare {@code x[0:size-1]} with {@code y[0:size-1]}, treating them as unsigned integers.
+   * @return -1, 0, or 1 depending on if {@code x<y}, {@code x==y}, or {@code x>y}.
+   * This is basically the same as gmp's {@code mpn_cmp} function.
    */
   public static int cmp (int[] x, int[] y, int size)
   {
@@ -465,15 +465,16 @@ class MPN
     return 0;
   }
 
-  /** Compare x[0:xlen-1] with y[0:ylen-1], treating them as unsigned integers.
-   * @result -1, 0, or 1 depending on if x<y, x==y, or x>y.
+  /** Compare {@code x[0:xlen-1]} with {@code y[0:ylen-1]}, treating them as unsigned integers.
+   * @return -1, 0, or 1 depending on
+   * whether {@code x<y}, {@code x==y}, or {@code x>y}.
    */
   public static int cmp (int[] x, int xlen, int[] y, int ylen)
   {
     return xlen > ylen ? 1 : xlen < ylen ? -1 : cmp (x, y, xlen);
   }
 
-  /* Shift x[x_start:x_start+len-1]count bits to the "right"
+  /* Shift x[x_start:x_start+len-1] count bits to the "right"
    * (i.e. divide by 2**count).
    * Store the len least significant words of the result at dest.
    * The bits shifted out to the right are returned.
@@ -498,6 +499,23 @@ class MPN
     return retval;
   }
 
+  /* Shift x[x_start:x_start+len-1] count bits to the "right"
+   * (i.e. divide by 2**count).
+   * Store the len least significant words of the result at dest.
+   * OK if dest==x.
+   * Assumes: 0 <= count < 32
+   * Same as rshift, but handles count==0 (and has no return value).
+   */
+  public static void rshift0 (int[] dest, int[] x, int x_start,
+			      int len, int count)
+  {
+    if (count > 0)
+      rshift(dest, x, x_start, len, count);
+    else
+      for (int i = 0;  i < len;  i++)
+	dest[i] = x[i + x_start];
+  }
+
   /** Return the long-truncated value of right shifting.
   * @param x a two's-complement "bignum"
   * @param len the number of significant words in x
@@ -522,20 +540,6 @@ class MPN
     return ((long)w1 << 32) | ((long)w0 & 0xffffffffL);
   }
 
-  /* Shift x[0:len-1]count bits to the "right" (i.e. divide by 2**count).
-   * Store the len least significant words of the result at dest.
-   * OK if dest==x.
-   * OK if count > 32 (but must be >= 0).
-   */
-  public static void rshift (int[] dest, int[] x, int len, int count)
-  {
-    int word_count = count >> 5;
-    count &= 31;
-    rshift (dest, x, word_count, len, count);
-    while (word_count < len)
-      dest[word_count++] = 0;
-  }
-
   /* Shift x[0:len-1] left by count bits, and store the len least
    * significant words of the result in dest[d_offset:d_offset+len-1].
    * Return the bits shifted out from the most significant digit.
@@ -616,8 +620,8 @@ class MPN
 
     // Temporarily devide both x and y by 2**sh.
     len -= initShiftWords;
-    MPN.rshift (x, x, initShiftWords, len, initShiftBits);
-    MPN.rshift (y, y, initShiftWords, len, initShiftBits);
+    MPN.rshift0 (x, x, initShiftWords, len, initShiftBits);
+    MPN.rshift0 (y, y, initShiftWords, len, initShiftBits);
 
     int[] odd_arg; /* One of x or y which is odd. */
     int[] other_arg; /* The other one can be even or odd. */
