@@ -1,4 +1,4 @@
-// Copyright (c) 2004  Per M.A. Bothner.
+// Copyright (c) 2004, 2007  Per M.A. Bothner.
 // This is free software;  for terms and warranty disclaimer see ./COPYING.
 
 package gnu.text;
@@ -23,10 +23,15 @@ import java.io.*;
   * @author Per Bothner <bothner@cygnus.com>
   */
 
-public class LineBufferedReader extends FilterReader
+public class LineBufferedReader extends Reader
 {
+  public void close () throws IOException { in.close(); }
+
+  // Not used by LineInputStreamReader subclass!
+  protected Reader in;
+
   /** Default (initial buffer) size. */
-  final static int BUFFER_SIZE = 1024;
+  public final static int BUFFER_SIZE = 8192;
 
   /** The input buffer, containing the current line etc. */
   public char[] buffer;
@@ -72,6 +77,22 @@ public class LineBufferedReader extends FilterReader
   /* If true in flags, char before start of buffer was '\r'. */
   private static final int PREV_WAS_CR = 4;
 
+  /** If true in flags, we don't need to keep the whole line.
+   * I.e. the application doesn't need to scan to the beginning of line. */
+  private static final int DONT_KEEP_FULL_LINES = 8;
+
+  /** Should we preserve the complete current line?
+   * The default is true, but in some cases there can be a performance
+   * improvement if we don't need to keep a long line when filling the buffer.
+   */
+  public void setKeepFullLines (boolean keep)
+  {
+    if (keep)
+      flags &= ~DONT_KEEP_FULL_LINES;
+    else
+      flags |= DONT_KEEP_FULL_LINES;
+  }
+
   /** True if CR and CRLF should be converted to LF. */
   public final boolean getConvertCR () { return (flags & CONVERT_CR) != 0; }
 
@@ -84,16 +105,17 @@ public class LineBufferedReader extends FilterReader
   }
 
   /** The position that marks the start of the current or marked line.
-    * If the readAheadLimit > 0 && markPos < pos, then it is the start of the
-    * line containing the markPos.
+    * If the {@code readAheadLimit > 0 && markPos < pos},
+    * then it is the start of the line containing the {@code markPos}.
     * If we are at the end of a line, and have not started reading the next
     * one (and are therefore allowed by unread back to the old line),
-    * the current line is still the old line; lineStartPos does not
+    * the current line is still the old line; {@code lineStartPos} does not
     * get set to the new pos until we read/peek the first char of the new line.
-    * If lineStartPos < 0, it means we went beyond the buffer maximum. */
-  int lineStartPos;
+    * If {@code lineStartPos < 0}, it means we went beyond the buffer maximum.
+    */
+  private int lineStartPos;
 
-  Object name;
+  Path path;
 
   /** The current line number (at position of lineStartPos). */
   protected int lineNumber;
@@ -103,18 +125,18 @@ public class LineBufferedReader extends FilterReader
     * (due to either calling reset or excessive reading ahead). */
   protected int readAheadLimit = 0;
 
-  /** The position of the mark (assuming readAheadLinit > 0).
-    * (Garbage if readAheadLimit <= 0). */
+  /** The position of the mark (assuming {@code readAheadLinit > 0}).
+    * (Garbage if {@code readAheadLimit <= 0}). */
   protected int markPos;
 
   public LineBufferedReader (InputStream in)
   {
-    super (new InputStreamReader(in));
+    this.in = new InputStreamReader(in);
   }
 
   public LineBufferedReader (Reader in)
   {
-    super (in);
+    this.in = in;
   }
 
   /** A hook to allow sub-classes to perform some action at start of line.
@@ -126,9 +148,9 @@ public class LineBufferedReader extends FilterReader
   {
   }
 
-  /** Called by read() when it needs its buffer filled.
+  /** Called by {@code read()} when it needs its buffer filled.
     * Read characters into buffer, starting at off, for len.
-    * Can assume that len > 0.  Only called if pos>=limit.
+    * Can assume that len > 0.  Only called if {@code pos>=limit}.
     * Return -1 if EOF, otherwise number of read chars.
     * This can be usefully overridden by sub-classes. */
   public int fill (int len) throws java.io.IOException
@@ -180,7 +202,7 @@ public class LineBufferedReader extends FilterReader
       }
   }
 
-  /* Make sure there is enough space for space more characters in buffer. */
+  /* Make sure there is enough space for more characters in buffer. */
 
   private void reserve (char[] buffer, int reserve)
     throws java.io.IOException
@@ -203,7 +225,9 @@ public class LineBufferedReader extends FilterReader
 	  }
 
 	reserve -= buffer.length;
-	if (saveStart < lineStartPos && reserve <= saveStart)
+	if (reserve <= saveStart
+            && (saveStart <= lineStartPos
+                || (flags & DONT_KEEP_FULL_LINES) != 0))
 	  ;
 	else if (reserve <= lineStartPos && saveStart > lineStartPos)
 	  saveStart = lineStartPos;
@@ -353,20 +377,24 @@ public class LineBufferedReader extends FilterReader
     return len;
   }
 
-  /** Should return a URI or a String. */
-  public Object getURI ()
+  public Path getPath ()
   {
-    return name;
+    return path;
+  }
+
+  public void setPath (Path path)
+  {
+    this.path = path;
   }
 
   public String getName ()
   {
-    return name == null ? null : name.toString();
+    return path == null ? null : path.toString();
   }
 
   public void setName (Object name)
   {
-    this.name = name;
+    setPath(Path.valueOf(name));
   }
 
   /** Get the current line number.
@@ -393,6 +421,12 @@ public class LineBufferedReader extends FilterReader
     this.lineNumber += lineNumber - getLineNumber();
   }
 
+  public void incrLineNumber (int lineDelta, int lineStartPos)
+  {
+    lineNumber += lineDelta;
+    this.lineStartPos = lineStartPos;
+  }
+
   /** Return the current (zero-based) column number. */
   public int getColumnNumber ()
   {
@@ -499,7 +533,7 @@ public class LineBufferedReader extends FilterReader
     int ch = read();
     if (ch < 0)
       return null;
-    if (ch == '\r' || ch == 'n')
+    if (ch == '\r' || ch == '\n')
       return "";
     int start = pos - 1;
     while (pos < limit)
@@ -507,6 +541,7 @@ public class LineBufferedReader extends FilterReader
 	ch = buffer[pos++];
 	if (ch == '\r' || ch == '\n')
           {
+            int end = pos - 1;
             if (ch != '\n' && ! getConvertCR())
               {
                 if (pos >= limit)
@@ -517,11 +552,11 @@ public class LineBufferedReader extends FilterReader
                 if (buffer[pos] == '\n')
                   pos++;
               }
-            return new String(buffer, start, pos - start);
+            return new String(buffer, start, end - start);
           }
       }
     StringBuffer sbuf = new StringBuffer(100);
-    sbuf.append(buffer, start, pos);
+    sbuf.append(buffer, start, pos - start);
     readLine(sbuf, 'I');
     return sbuf.toString();
   }
@@ -689,5 +724,4 @@ public class LineBufferedReader extends FilterReader
       unread_quick();
     return c;
   }
-
 }
