@@ -3,26 +3,32 @@
 
 package gnu.xml;
 import gnu.lists.*;
-import gnu.text.Char;
-import gnu.text.SourceLocator;
-import gnu.text.SourceMessages;
+import gnu.text.*;
 import gnu.mapping.Symbol;
+/* #ifdef SAX2 */
+import org.xml.sax.*;
+import gnu.kawa.sax.*;
+/* #endif */
 import gnu.expr.Keyword; // FIXME - bad cross-package dependency.
 
 /** Fixup XML input events.
  * Handles namespace resolution, and adds "namespace nodes" if needed.
  * Does various error checking.
  * This wrapper should be used when creating a NodeTree,
- * has is done XQuery node constructor expressions.
+ * as is done for XQuery node constructor expressions.
  * Can also be called directly from XMLParser, in which case we use a slightly
  * lower-level interface where we array char array segments rather than
  * Strings.  This is to avoid duplicate String allocation and interning.
  * The combination XMLParser+XMLFilter+NodeTree makes for a fast and
  * compact way to read an XML file into a DOM.
- * Future: Subsume ConsumeSAXHandler as well.  FIXME
  */
 
-public class XMLFilter implements XConsumer, PositionConsumer
+public class XMLFilter implements
+                       /* #ifdef SAX2 */
+                       DocumentHandler, ContentHandler,
+                       /* #endif */
+                       gnu.text.SourceLocator,
+                       XConsumer, PositionConsumer
 {
   /** This is where we save attributes while processing a begin element.
    * It may be the final output if {@code out instanceof NodeTree}. */
@@ -42,37 +48,39 @@ public class XMLFilter implements XConsumer, PositionConsumer
    * This is logically multiple separate stacks, but we combine them into
    * a single array.  While this makes the code a little harder to read,
    * it reduces memory overhead and (more importantly) should improve locality.
-   * For each nested document or group there is the saved value of
+   * For each nested document or element there is the saved value of
    * namespaceBindings followed by a either a MappingInfo or Symbol
-   * from the emitBeginElement/beginGroup.  This is followed by a MappingInfo
-   * or Symbol for each attribute we seen for the current group. */
+   * from the emitBeginElement/startElement.  This is followed by a MappingInfo
+   * or Symbol for each attribute we seen for the current element. */
   Object[] workStack;
   NamespaceBinding namespaceBindings;
 
   private SourceMessages messages;
   SourceLocator locator;
+  LineBufferedReader in;
 
+  public void setSourceLocator (LineBufferedReader in)
+  { this.in = in;  this.locator = this; }
   public void setSourceLocator (SourceLocator locator)
   { this.locator = locator; }
   public void setMessages (SourceMessages messages)
   { this.messages = messages; }
 
-  /** Twice the number of active beginGroup and beginDocument calls. */
+  /** Twice the number of active startElement and startDocument calls. */
   protected int nesting;
 
   int previous = 0;
-  private static final int SAW_CR = 1;
-  private static final int SAW_KEYWORD = 2;
-  private static final int SAW_WORD = 3;
+  private static final int SAW_KEYWORD = 1;
+  private static final int SAW_WORD = 2;
 
   /** If {@code stringizingLevel > 0} then stringize rather than copy nodes.
-   * It counts the number of nested beginAttributes that are active.
+   * It counts the number of nested startAttributes that are active.
    * (In the future it should also count begun comment and
    * processing-instruction constructors, when those support nesting.) */
   protected int stringizingLevel;
-  /** Value of {@code nesting} just before outermost beginGroup
+  /** Value of {@code nesting} just before outermost startElement
    * while {@code stringizingLevel > 0}.
-   * I.e. if we're nested inside a group nested inside an attribute
+   * I.e. if we're nested inside a element nested inside an attribute
    * then {@code stringizingElementNesting >= 0},
    * otherwise {@code stringizingElementNesting == -1}. */
   protected int stringizingElementNesting = -1;
@@ -119,11 +127,6 @@ public class XMLFilter implements XConsumer, PositionConsumer
 
   boolean mismatchReported;
 
-  public void setLocator (SourceLocator locator)
-  {
-    this.locator = locator;
-  }
-
   /** Functionally equivalent to
    * {@code new NamespaceBinding(prefix, uri, oldBindings},
    * but uses "hash consing".
@@ -326,9 +329,7 @@ public class XMLFilter implements XConsumer, PositionConsumer
     if (uri != null)
       return uri;
     if (prefix != null)
-      {
-        error('e', "unknown namespace prefix '" + prefix + '\'');
-      }
+      error('e', "unknown namespace prefix '" + prefix + '\'');
     return "";
   }
 
@@ -343,7 +344,7 @@ public class XMLFilter implements XConsumer, PositionConsumer
       endAttribute();
     NamespaceBinding outer = nesting == 0 ? NamespaceBinding.predefinedXML
       : (NamespaceBinding) workStack[nesting-2];
-      
+
     NamespaceBinding bindings = namespaceBindings;
 
     // This first pass is to check that there are namespace declarations for
@@ -556,7 +557,7 @@ public class XMLFilter implements XConsumer, PositionConsumer
 		info.index = index;
 	      }
 	    if (i == 0)
-              tlist.setGroupName(tlist.gapEnd, index);
+              tlist.setElementName(tlist.gapEnd, index);
 	    else if (! isNsNode || namespacePrefixes)
               tlist.setAttributeName(startIndexes[i-1], index);
 	  }
@@ -565,10 +566,10 @@ public class XMLFilter implements XConsumer, PositionConsumer
             Object type = info == null ? saved
               : i == 0 ? info.type : info.qname;
 	    if (i == 0)
-	      out.beginGroup(type);
+	      out.startElement(type);
 	    else if (! isNsNode || namespacePrefixes)
 	      {
-		out.beginAttribute(type);
+		out.startAttribute(type);
 		int start = startIndexes[i-1];
 		int end = i < attrCount ? startIndexes[i] : tlist.gapStart;
 		tlist.consumeIRange(start + TreeList.BEGIN_ATTRIBUTE_LONG_SIZE,
@@ -578,13 +579,19 @@ public class XMLFilter implements XConsumer, PositionConsumer
 	      }
 	  }
       }
+
+    /* #ifdef SAX2 */
+    if (out instanceof ContentConsumer)
+      ((ContentConsumer) out).endStartTag();
+    /* #endif */
+      
     for (int i = 1;  i <= attrCount; i++)
       workStack[nesting+i-1] = null; // For GC.
     if (out != tlist)
       {
-	base = out;
-	// Remove temporarily stored attributes.
-	tlist.clear();
+        base = out;
+        // Remove temporarily stored attributes.
+        tlist.clear();
       }
     attrCount = -1;
   }
@@ -675,7 +682,7 @@ public class XMLFilter implements XConsumer, PositionConsumer
     else if (v instanceof Keyword)
       {
         Keyword k = (Keyword) v;
-        beginAttribute(k.asSymbol());
+        startAttribute(k.asSymbol());
         previous = SAW_KEYWORD;
       }
     else
@@ -734,120 +741,46 @@ public class XMLFilter implements XConsumer, PositionConsumer
       base.write(str, start, length);
   }
 
+  final boolean inElement ()
+  {
+    int i = nesting;
+    // Don't count nested document nodes.
+    while (i > 0 && workStack[i-1] == null)
+      i -= 2;
+    return i != 0;
+  }
+
+  public void linefeedFromParser ()
+  {
+    if (inElement() && checkWriteAtomic())
+      base.write('\n');
+  }
+
   public void textFromParser (char[] data, int start, int length)
   {
-    // Skip whitespace not in an element.
-    // This works semi-accidentally, since XMLParser doesn't call beginDocument
-    // which otherwise would increment nesting.  Perhaps shipping toplevel
-    // whitespace should be handled internally in XMLParser.  FIXME.
-    if (nesting == 0)
+    // We skip whitespace not in an element.
+    if (! inElement())
       {
         for (int i = 0; ; i++)
           {
             if (i == length)
               return;
             if (! Character.isWhitespace(data[start+i]))
-              break;
+              {
+                error('e', "text at document level");
+                break;
+              }
           }
       }
     else if (length > 0)
       {
-        if (previous == SAW_CR)
-          {
-            char ch = data[start];
-            previous = 0;
-            if (ch == '\n' || ch == 0x85)
-              {
-                start++;
-                length--;
-              }
-          }
         if (! checkWriteAtomic())
           return;
 
-        // The complication here is line-end normalization,
-        // with minimal overhead.
-        int limit = start + length;
-        TreeList blist = base instanceof TreeList ? (TreeList) base : null;
-      outerLoop:
-        for (int i = start;  ;  i++)
-          {
-            char ch;
-            // We optimize the case that base instanceof TreeList.
-            if (blist != null)
-              {
-                blist.ensureSpace(limit-i);
-                char[] bdata = blist.data;
-                int gapStart = blist.gapStart;
-                for (;; i++)
-                  {
-                    if (i >= limit)
-                      {
-                        blist.gapStart = gapStart;
-                        break outerLoop;
-                      }
-                    ch = data[i];
-                    if (ch != '\r' && ch < 0x85) // Quick trest first.
-                      bdata[gapStart++] = ch;
-                    else if (ch > TreeList.MAX_CHAR_SHORT)
-                      {
-                        blist.gapStart = gapStart;
-                        blist.write(ch);
-                        continue outerLoop;
-                      }
-                    else if (ch == '\r' || ch == 0x85 || ch == 0x2028)
-                      {
-                        blist.gapStart = gapStart;
-                        start = i+1;
-                        break;
-                      }
-                    else
-                      bdata[gapStart++] = ch;
-                  }
-              }
-            else if (i >= limit)
-              ch = 0;
-            else
-              {
-                ch = data[i];
-                if (ch >= ' ' && ch < 0x85) // Quick(er) test.
-                  continue;
-                if (ch != '\r' && ch != 0x85 && ch != 0x2028)
-                  continue;
-              }
-            if (i > start)
-              {
-                base.write(data, start, i-start);
-              }
-            if (i >= limit)
-              break;
-            start = i+1;
-            if (ch == '\r')
-              {
-                if (start < limit)
-                  {
-                    ch = data[i+1];
-                    if (ch == '\n')
-                      continue; // Handled next iteration.
-                    if (ch == 0x85)
-                      i++;
-                  }
-                else
-                  previous = SAW_CR;
-              }
-            base.write('\n');
-          }
+        base.write(data, start, length);
       }
   }
 
-  public void write (String str, int start, int length)
-  {
-    if (length == 0)
-      writeJoiner();
-    else if (checkWriteAtomic())
-      base.write(str, start, length);
-  }
-
   protected void writeJoiner ()
   {
     previous = 0;
@@ -870,14 +803,14 @@ public class XMLFilter implements XConsumer, PositionConsumer
       }
   }
 
-  protected void beginGroupCommon ()
+  protected void startElementCommon ()
   {
     closeStartTag();
     if (stringizingLevel == 0)
       {
         ensureSpaceInWorkStack(nesting);
         workStack[nesting] = namespaceBindings;
-        tlist.beginGroup(0);
+        tlist.startElement(0);
         base = tlist;
         attrCount = 0;
       }
@@ -893,18 +826,18 @@ public class XMLFilter implements XConsumer, PositionConsumer
   }
 
   /** Process a start tag, with the given element name. */
-  public void emitBeginElement(char[] data, int start, int count)
+  public void emitStartElement(char[] data, int start, int count)
   { 
     closeStartTag();
     MappingInfo info = lookupTag(data, start, count);
-    beginGroupCommon();
+    startElementCommon();
     ensureSpaceInWorkStack(nesting-1);
     workStack[nesting-1] = info;
   }
 
-  public void beginGroup(Object type)
+  public void startElement(Object type)
   {
-    beginGroupCommon();
+    startElementCommon();
     if (stringizingLevel == 0)
       {
         ensureSpaceInWorkStack(nesting-1);
@@ -935,7 +868,7 @@ public class XMLFilter implements XConsumer, PositionConsumer
               }
             if (inherited == null)
               {
-                // This is the outer-most group.
+                // This is the outer-most element.
                 namespaceBindings
                   = (type instanceof XName ? ((XName) type).getNamespaceNodes()
                      : NamespaceBinding.predefinedXML);
@@ -977,7 +910,7 @@ public class XMLFilter implements XConsumer, PositionConsumer
     return findNamespaceBinding(prefix, uri, list);
   }
 
-  private boolean beginAttributeCommon()
+  private boolean startAttributeCommon()
   {
     if (stringizingElementNesting >= 0)
       ignoringLevel++;
@@ -993,7 +926,7 @@ public class XMLFilter implements XConsumer, PositionConsumer
     return true;
   }
 
-  public void beginAttribute (Object attrType)
+  public void startAttribute (Object attrType)
   {
     previous = 0;
     if (attrType instanceof Symbol)
@@ -1011,25 +944,25 @@ public class XMLFilter implements XConsumer, PositionConsumer
       error('e', "attribute not allowed at document level");
     if (attrCount < 0 && nesting > 0)
       error('e', "attribute '"+attrType+"' follows non-attribute content");
-    if (! beginAttributeCommon())
+    if (! startAttributeCommon())
       return;
     workStack[nesting+attrCount-1] = attrType;
     if (nesting == 0)
-      base.beginAttribute(attrType);
+      base.startAttribute(attrType);
     else
-      tlist.beginAttribute(0);
+      tlist.startAttribute(0);
   }
 
   /** Process an attribute, with the given attribute name.
    * The attribute value is given using {@code write}.
-   * The value is terminated by either another emitBeginAttribute
+   * The value is terminated by either another emitStartAttribute
    * or an emitEndAttributes.
    */
-  public void emitBeginAttribute(char[] data, int start, int count)
+  public void emitStartAttribute (char[] data, int start, int count)
   {
     if (attrLocalName != null)
       endAttribute();
-    if (! beginAttributeCommon())
+    if (! startAttributeCommon())
       return;
 
     MappingInfo info = lookupTag(data, start, count);
@@ -1053,7 +986,7 @@ public class XMLFilter implements XConsumer, PositionConsumer
 	  }
       }
     if (currentNamespacePrefix == null || namespacePrefixes)
-      tlist.beginAttribute(0);
+      tlist.startAttribute(0);
   }
 
   /** Process the end of a start tag.
@@ -1062,6 +995,7 @@ public class XMLFilter implements XConsumer, PositionConsumer
   {
     if (attrLocalName != null)
       endAttribute();
+    closeStartTag();
   }
 
   /** Process an end tag.
@@ -1074,7 +1008,7 @@ public class XMLFilter implements XConsumer, PositionConsumer
 	error('e', "unclosed attribute"); // FIXME
 	endAttribute();
       }
-    if (nesting == 0)
+    if (! inElement())
       {
 	error('e', "unmatched end element"); // FIXME
 	return;
@@ -1107,10 +1041,10 @@ public class XMLFilter implements XConsumer, PositionConsumer
     closeStartTag();
     if (nesting <= 0)
       return; // Only if error.
-    endGroup();
+    endElement();
   }
 
-  public void endGroup ()
+  public void endElement ()
   {
     closeStartTag();
     nesting -= 2;
@@ -1120,7 +1054,7 @@ public class XMLFilter implements XConsumer, PositionConsumer
         namespaceBindings = (NamespaceBinding) workStack[nesting];
         workStack[nesting] = null;
         workStack[nesting+1] = null;
-        base.endGroup();
+        base.endElement();
       }
     else if (stringizingElementNesting == nesting)
       {
@@ -1268,7 +1202,7 @@ public class XMLFilter implements XConsumer, PositionConsumer
                                         int dstart, int dlength)
   {
     // Skip XML declaration.
-    if (nesting == 0 && tlength == 3
+    if (tlength == 3 && ! inElement()
         && buffer[tstart] == 'x'
         && buffer[tstart+1] == 'm'
         && buffer[tstart+2] == 'l')
@@ -1277,7 +1211,7 @@ public class XMLFilter implements XConsumer, PositionConsumer
     processingInstructionCommon(target, buffer, dstart, dlength);
   }
 
-  public void beginDocument()
+  public void startDocument()
   {
     closeStartTag();
     if (stringizingLevel > 0)
@@ -1286,7 +1220,7 @@ public class XMLFilter implements XConsumer, PositionConsumer
     else
       {
         if (nesting == 0)
-          base.beginDocument();
+          base.startDocument();
         else
           writeJoiner();
         ensureSpaceInWorkStack(nesting);
@@ -1496,7 +1430,7 @@ public class XMLFilter implements XConsumer, PositionConsumer
   }
 
   public static String
-  duplicateAttributeMessage (Symbol attrSymbol, Object groupName)
+  duplicateAttributeMessage (Symbol attrSymbol, Object elementName)
   {
     StringBuffer sbuf = new StringBuffer("duplicate attribute: ");
     String uri = attrSymbol.getNamespaceURI();
@@ -1507,10 +1441,10 @@ public class XMLFilter implements XConsumer, PositionConsumer
         sbuf.append(uri);
       }
     sbuf.append(attrSymbol.getLocalPart());
-    if (groupName != null)
+    if (elementName != null)
       {
         sbuf.append(" in <");
-        sbuf.append(groupName);
+        sbuf.append(elementName);
         sbuf.append('>');
       }
     return sbuf.toString();
@@ -1530,6 +1464,129 @@ public class XMLFilter implements XConsumer, PositionConsumer
   {
     return ignoringLevel > 0;
   }
+
+  /* #ifdef SAX2 */
+  public void setDocumentLocator(Locator locator)
+  {
+    if (locator instanceof SourceLocator)
+      this.locator = (SourceLocator) locator;
+    else
+      ; // create SourceLocator that indirects to given locator.  FIXME.
+  }
+
+  public void startElement (String namespaceURI, String localName,
+                            String qName, Attributes atts)
+  {
+    startElement(Symbol.make(namespaceURI, localName));
+    int numAttributes = atts.getLength();
+    for (int i = 0;  i < numAttributes;  i++)
+      {
+        startAttribute(Symbol.make(atts.getURI(i), atts.getLocalName(i)));
+        write(atts.getValue(i));
+        endAttribute();
+      }
+  }
+
+  public void endElement (String namespaceURI, String localName,
+                          String qName)
+  {
+    endElement();
+  }
+
+  public void startElement (String name, AttributeList atts)
+  {
+    name = name.intern();  // ???
+    startElement(name);  // FIXME
+    int attrLength = atts.getLength();
+    for (int i = 0; i < attrLength; i++)
+      {
+        name = atts.getName(i);
+        name = name.intern();  // ?
+        String type = atts.getType(i);
+        String value = atts.getValue(i);
+        startAttribute(name);  // FIXME
+        write(value);
+        endAttribute();
+      }
+  }
+
+  public void endElement (String name)
+    throws SAXException
+  {
+    endElement();
+  }
+
+  public void characters (char ch[], int start, int length)
+    throws SAXException
+  {
+    write(ch, start, length);
+  }
+
+  public void ignorableWhitespace (char ch[], int start, int length)
+    throws SAXException
+  {
+    write(ch, start, length); // FIXME
+  }
+  /* #endif */
+
+  public void processingInstruction(String target, String data)
+  {
+    // FIXME - this could be done more efficiently by copying the
+    // string into tlist.
+    char[] chars = data.toCharArray();
+    processingInstructionCommon(target, chars, 0, chars.length);
+  }
+
+  public void startPrefixMapping (String prefix, String uri)
+  {
+    namespaceBindings = findNamespaceBinding(prefix.intern(),
+                                             uri.intern(),
+                                             namespaceBindings);
+  }
+
+  public void endPrefixMapping (String prefix)
+  {
+    namespaceBindings = namespaceBindings.getNext();
+  }
+
+  public void skippedEntity (String name)
+  {
+    // FIXME
+  }
+
+  public String getPublicId ()
+  {
+    return null;
+  }
+
+  public String getSystemId ()
+  {
+    return in == null ? null : in.getName();
+  }
+
+  public String getFileName ()
+  {
+    return in == null ? null : in.getName();
+  }
+
+  public int getLineNumber()
+  {
+    if (in == null)
+      return -1;
+    int line = in.getLineNumber();
+    return line < 0 ? -1 : line + 1;
+  }
+
+  public int getColumnNumber()
+  {
+    int col;
+    return in != null && (col = in.getColumnNumber()) > 0 ? col : -1;
+  }
+
+  public boolean isStableSourceLocation()
+  {
+    return false;
+  }
 }
 
 final class MappingInfo
