@@ -10,6 +10,7 @@ import gnu.kawa.xml.KNode;
 import gnu.kawa.xml.UntypedAtomic;
 import java.util.regex.Pattern;
 import java.util.regex.Matcher;
+import gnu.text.*;
 /* #ifdef use:java.text.Normalizer */
 // import java.text.Normalizer;
 /* #endif */
@@ -26,6 +27,7 @@ public class StringUtils
     if ((arg == Values.empty || arg == null) && onEmpty != ERROR_VALUE)
       return onEmpty;
     if (arg instanceof UntypedAtomic 
+        || arg instanceof Path
         /* #ifdef use:java.net.URI */
         || arg instanceof java.net.URI
         /* #endif */
@@ -370,7 +372,6 @@ public class StringUtils
 
   static String encodeForUri (Object arg, char mode)
   {
-    StringBuffer sbuf = new StringBuffer();
     String str;
     if (arg instanceof String || arg instanceof UntypedAtomic)
       str = arg.toString();
@@ -378,62 +379,7 @@ public class StringUtils
       str = "";
     else
       throw new ClassCastException();
-    int len = str.length();
-    for (int i = 0; i <len;  i++)
-      {
-        int ch = str.charAt(i);
-        // FIXME: Check for surrogate.
-        if (mode == 'H' ? ch >= 32 && ch <= 126
-            : ((ch >= 'a' && ch <= 'z') || (ch >= 'A' && ch <= 'Z')
-               || (ch >= '0' && ch <= '9')
-               || ch == '-' || ch == '_' || ch == '.' || ch == '~'
-               || (mode == 'I'
-                   && (ch == ';' || ch == '/' || ch == '?' || ch == ':'
-                       || ch == '*' || ch == '\'' || ch == '(' || ch == ')'
-                       || ch == '@' || ch == '&' || ch == '=' || ch == '+'
-                       || ch == '$' || ch == ',' || ch == '[' || ch == ']'
-                       || ch == '#' || ch == '!' || ch == '%'))))
-          sbuf.append((char) ch);
-        else
-          {
-            int pos = sbuf.length();
-            int nbytes = 0;
-            int needed = ch < (1 << 7) ? 1
-              : ch < (1 << 11) ? 2
-              : ch < (1 << 16) ? 3
-              : 4;
-            do
-              {
-                // We insert encodings for the bytes in right-to-left order.
-                int availbits = nbytes == 0 ? 7 : 6 - nbytes;
-                int b;
-                if (ch < (1 << availbits))
-                  {
-                    // The rest fits: handling first bytes.
-                    b = ch;
-                    if (nbytes > 0)
-                      b |= (0xff80 >> nbytes) & 0xff;
-                    ch = 0;
-                  }
-                else
-                  {
-                    b = 0x80 | (ch & 0x3f);
-                    ch >>= 6;
-                  }
-                nbytes++;
-                for (int j = 0; j <= 1; j++)
-                  {
-                    int hex = b & 15;
-                    sbuf.insert(pos,
-                                (char) (hex <= 9 ? hex + '0' : hex - 10 + 'A'));
-                    b >>= 4;
-                  }
-                sbuf.insert(pos, '%');
-              }
-            while (ch != 0);
-          }
-      }
-    return sbuf.toString();
+    return URIPath.encodeForUri(str, mode);
   }
 
   public static String normalizeSpace (Object arg)
