@@ -4,13 +4,65 @@
 (require <kawa.lib.syntax>)
 (require <kawa.lib.ports>)
 
-(define (file-exists? filename)
-  ((primitive-virtual-method "java.io.File" "exists" "boolean" ())
-   (->pathname filename)))
+(define (path? path) :: <boolean>
+  (instance? path <gnu.text.Path>))
+(define (filepath? path) :: <boolean>
+  (instance? path <gnu.text.FilePath>))
+(define (URI? path) :: <boolean>
+  (instance? path <gnu.text.URIPath>))
+(define (absolute-path? (path :: path)) :: <boolean>
+  (path:isAbsolute))
+(define (path-scheme (p :: path))
+  (let ((s (p:getScheme)))
+    (if (eq? s #!null) #f (<string> s))))
+(define (path-authority (p :: path))
+  (let ((s (p:getAuthority)))
+    (if (eq? s #!null) #f (<string> s))))
+(define (path-user-info (p :: path))
+  (let ((s (p:getUserInfo)))
+    (if (eq? s #!null) #f (<string> s))))
+(define (path-host (p :: path))
+  (p:getHost))
+(define (path-file (p :: path))
+  (let ((s (p:getPath)))
+    (if (eq? s #!null) #f (<string> s))))
+(define (path-directory (p :: path))
+  (let ((s (p:getDirectory)))
+    (if (eq? s #!null) #f (<string> s))))
+(define (path-parent (p :: path))
+  (let ((s (p:getParent)))
+    (if (eq? s #!null) #f (<string> s))))
+(define (path-last (p :: path))
+  (let ((s (p:getLast)))
+    (if (eq? s #!null) #f (<string> s))))
+(define (path-extension (p :: path))
+  (let ((s (p:getExtension)))
+    (if (eq? s #!null) #f (<string> s))))
+(define (path-port (p :: path)) :: <int>
+  (p:getPort))
+(define (path-query (p :: path))
+  (let ((s (p:getQuery)))
+    (if (eq? s #!null) #f (<string> s))))
+(define (path-fragment (p :: path))
+  (let ((s (p:getFragment)))
+    (if (eq? s #!null) #f (<string> s))))
 
-(define (file-directory? filename)
-  ((primitive-virtual-method "java.io.File" "isDirectory" "boolean" ())
-   (->pathname filename)))
+#|
+(resolve-path path) ;; resolves symlinks
+(path->complete-path path [base-path])
+(path->directory-path path)
+(string->path string)
+(path->string path)
+(build-path base-path sub-path ...)
+(expand-path)
+(simplify-path)
+|#
+
+(define (file-exists? (filename :: path)) :: <boolean>
+  (filename:exists))
+
+(define (file-directory? (filename :: path)) :: <boolean>
+  (filename:isDirectory))
 
 (define (file-readable? filename)
   ((primitive-virtual-method "java.io.File" "canRead" "boolean" ())
@@ -20,9 +72,8 @@
   ((primitive-virtual-method "java.io.File" "canWrite" "boolean" ())
    (->pathname filename)))
 
-;(define (file-modification-time filename)
-;  ((primitive-virtual-method "java.io.File" "lastModified" "long" ())
-;   (->pathname filename)))
+;(define (file-modification-time (filename :: path)) :: <long>
+;  (filename:getLastModified))
 
 (define (delete-file filename)
   ((primitive-virtual-method "java.io.File" "delete" "boolean" ())
@@ -49,9 +100,8 @@
    (->pathname dirname)))
 
 ; Taken from MIT Scheme
-(define (->pathname filename)
-  ((primitive-constructor "java.io.File" ("String"))
-    filename))
+(define (->pathname filename) :: path
+  (path filename))
   
 (define (%file-separator)
   (make <string>
@@ -68,11 +118,8 @@
 ; From scsh
 ;(define (directory-files [dir [dotfiles?]]) ...)
 
-(define (URI url) :: <URI>
-  (gnu.text.URI_utils:toURI url))
-
-(define (resolve-uri uri base) :: <URI>
-  (gnu.text.URI_utils:resolve uri base))
+(define (resolve-uri (uri :: path) (base :: path)) :: path
+  (base:resolve uri))
 
 (define-syntax module-uri
   (lambda (form)
@@ -84,7 +131,7 @@
 (define-syntax resource-uri
   (syntax-rules ()
     ((resource-uri uri)
-     (gnu.text.URI_utils:resolve uri (module-uri)))))
+     ((module-uri):resolve uri))))
 
 ; From MzLib.  Scsh has (create-temp-file [prefix]).
 (define (make-temporary-file #!optional (fmt "kawa~d.tmp")) :: <string>
@@ -121,16 +168,15 @@
 (define-syntax (include-relative x)
   (syntax-case x ()
 	       ((_ filename)
-		(let ((path-pair :: <pair>
-				 (syntax-object->datum (syntax (filename)))))
+		(let* ((path-pair :: <gnu.lists.PairWithPosition>
+				  (syntax-object->datum (syntax (filename))))
+		       (base :: path (path-pair:getFileName))
+		       (fname ((field path-pair 'car):toString)))
 		  (list
 		   (datum->syntax-object (syntax filename) 'include)
 		   (datum->syntax-object
 		    (syntax filename)
-		    (make <string>
-		     (gnu.text.URI_utils:resolve
-		      (field path-pair 'car)
-		      (gnu.lists.PairWithPosition:getFileName path-pair)))))))))
+		    (make <string> ((base:resolve fname):toString))))))))
 
 #|
 (define-syntax source-file
