@@ -1,5 +1,11 @@
+(require <kawa.lib.prim_syntax>)
+
 ;;; Definitions for some standard syntax.
 
+(module-export cond case and or let let* letrec do delay
+	       syntax-object->datum datum->syntax-object with-syntax
+	       generate-temporaries)
+
 ;;; COND
 
 (define-syntax cond
@@ -12,11 +18,11 @@
 		 (%syntax-error "else clause must be last clause of cond"))
 
 		((cond (test => result))
-		 (let ((temp test))
+		 (%let ((temp test))
 		   (if temp (result temp))))
 
 		((cond (test => result) clause1 clause2 ...)
-		 (let ((temp test))
+		 (%let ((temp test))
 		   (if temp
 		       (result temp)
 		       (cond clause1 clause2 ...))))
@@ -30,7 +36,7 @@
 		((cond (test result1 result2 ...))
 		 (if test (begin result1 result2 ...)))
 
-		 ((cond (test result1 result2 ...) clause1 clause2 ...)
+		((cond (test result1 result2 ...) clause1 clause2 ...)
 		 (if test
 		     (begin result1 result2 ...)
 		     (cond clause1 clause2 ...)))))
@@ -39,7 +45,7 @@
 
 (define-syntax case (syntax-rules ()
 				  ((case key clauses ...)
-				   (let ((tmp key))
+				   (%let ((tmp key))
 				     (%case tmp clauses ...)))))
 
 (define-syntax %case (syntax-rules (else)
@@ -66,14 +72,32 @@
 					  (or (eqv? key 'datum)
 					      (%case-match key more ...)))))
 
+(define-syntax %lang-boolean
+  (syntax-rules ()
+    ((%lang-boolean value)
+     (make <gnu.expr.QuoteExp>
+       (invoke
+	(gnu.expr.Language:getDefaultLanguage)
+	'booleanObject value)))))
+
 ;;; AND
 
-(define-syntax and
-  (syntax-rules ()
-		((and) #t)
-		((and test) test)
-		((and test1 test2 test3 ...)
-		 (and (if test1 test2 #f) test3 ...))))
+(define-syntax (and f)
+  (syntax-case f ()
+	       ((and) (%lang-boolean #t))
+	       ((and test) (syntax test))
+	       ((and test1 test2 ...)
+		(syntax (%let ((x test1))
+			  (if x (and test2 ...) x))))))
+;;; OR
+
+(define-syntax (or f)
+  (syntax-case f ()
+	       ((or) (%lang-boolean #f))
+	       ((or test) (syntax test))
+	       ((or test1 test2 ...)
+		(syntax (%let ((x test1))
+			  (if x x (or test2 ...)))))))
 
 ;;; LET (including named let)
 
@@ -112,33 +136,50 @@
 
 (define-syntax let
   (syntax-rules ()
-		((let (binding ...) . body)
-		 (%let (binding ...) . body))
-		; Alternative definition would be simpler, but makes more
-		; work for compiler to optimize it - and still doesn't
-		; do quite as well.
-		;((%let-lambda1 (binding ...) () body)
-		;(%let-init binding) ...))
-		((let name (binding ...) . body)
-		 ((letrec ((name (%let-lambda1 (binding ...) () body)))
-		    name)
-		  (%let-init binding) ...))))
+    ((let (binding ...) . body)
+     ;; We destructure the bindings to simplify %let's job.
+     (%let (binding ...) . body))
+    ;; Alternative definition would be simpler, but makes more work for
+    ;; the compiler to optimize it - and still doesn't do quite as well.
+    ;;((%let-lambda1 (binding ...) () body)
+    ;;(%let-init binding) ...))
+    ((let name (binding ...) . body)
+     ((letrec ((name (%let-lambda1 (binding ...) () body)))
+	name)
+      (%let-init binding) ...))))
 
 ;;; LET*
 
-(define-syntax let* (syntax-rules ()
-				 ((let* () . body) (%let () . body))
-				 ((let* (var-init) . body)
-				  (%let (var-init) . body))
-				 ((let* (var-init . bindings) . body)
-				  (%let (var-init)
-				    (let* bindings . body)))
-				 ((let* bindings . body)
-				  (%syntax-error
-				   "invalid bindings list in let*"))
-				 ((let* . body)
-				  (%syntax-error
-				   "missing bindings list in let*"))))
+(define-syntax let*
+  (syntax-rules ()
+    ((let* () . body) (%let () . body))
+    ((let* (var-init) . body)
+     (%let (var-init) . body))
+    ((let* (var-init . bindings) . body)
+     (%let (var-init)
+	   (let* bindings . body)))
+    ((let* bindings . body)
+     (%syntax-error
+      "invalid bindings list in let*"))
+    ((let* . body)
+     (%syntax-error
+      "missing bindings list in let*"))))
+
+;;; LETREC
+
+(define-syntax letrec
+  (syntax-rules ()
+    ((letrec bindings . body)
+     (%letrec1 () bindings . body))))
+
+(define-syntax %letrec1
+  (syntax-rules (::)
+    ((%letrec1 done ((x :: type init) . bindings) . body)
+     (%letrec1 ((x :: type #!undefined) . done) bindings (set! x init) . body))
+    ((%letrec1 done ((x init) . bindings) . body)
+     (%letrec1 ((x #!undefined) . done) bindings (set! x init) . body))
+    ((%letrec1 done () . body)
+     (%let done . body))))
 
 ;;; DO
 
@@ -175,9 +216,42 @@
 
 ;;; DELAY
 
-(define (%make-promise x)
-  ((primitive-constructor <kawa.lang.Promise> (<gnu.mapping.Procedure>)) x))
-
 (define-syntax delay (syntax-rules ()
 				   ((delay expression)
-				    (%make-promise (lambda () expression)))))
+				    (make <kawa.lang.Promise> (lambda () expression)))))
+
+(define (syntax-object->datum obj)
+  (kawa.lang.Quote:quote obj))
+
+(define (datum->syntax-object template-identifier obj)
+  (kawa.lang.SyntaxForm:makeWithTemplate template-identifier obj))
+
+(define (generate-temporaries list)
+  (let loop ((n (kawa.lang.Translator:listLength list)) (lst '()))
+    (if (= n 0) lst
+	(loop (- n 1) (make <pair> (datum->syntax-object list (gnu.expr.Symbols:gentemp)) lst)))))
+
+;;; The definition of include is based on that in the portable implementation
+;;; of syntax-case psyntax.ss, whixh is again based on Chez Scheme.
+;;; Copyright (c) 1992-2002 Cadence Research Systems
+;;; Permission to copy this software, in whole or in part, to use this
+;;; software for any lawful purpose, and to redistribute this software
+;;; is granted subject to the restriction that all copies made of this
+;;; software must include this copyright notice in full.  This software
+;;; is provided AS IS, with NO WARRANTY, EITHER EXPRESS OR IMPLIED,
+;;; INCLUDING BUT NOT LIMITED TO IMPLIED WARRANTIES OF MERCHANTABILITY
+;;; OR FITNESS FOR ANY PARTICULAR PURPOSE.  IN NO EVENT SHALL THE
+;;; AUTHORS BE LIABLE FOR CONSEQUENTIAL OR INCIDENTAL DAMAGES OF ANY
+;;; NATURE WHATSOEVER.
+
+;; Converted to use syntax-rules from the psyntax.ss implementation.
+
+(define-syntax with-syntax
+   (syntax-rules ()
+     ((with-syntax () e1 e2 ...)
+      (begin e1 e2 ...))
+     ((with-syntax ((out in)) e1 e2 ...)
+      (syntax-case in () (out (begin e1 e2 ...))))
+     ((with-syntax ((out in) ...) e1 e2 ...)
+      (syntax-case (list in ...) ()
+		   ((out ...) (begin e1 e2 ...))))))
