@@ -1,12 +1,12 @@
+(require <kawa.lib.prim_syntax>)
+
 (define-syntax %do-trace
   (syntax-rules ()
 		((%do-trace proc flag)
 		 (set! proc
-		       ((primitive-static-method
-			 <kawa.standard.TracedProcedure> "doTrace"
-			 <kawa.standard.TracedProcedure>
-			 (<function> <boolean>))
-			proc flag)))))
+		       (invoke-static
+			 <kawa.standard.TracedProcedure> 'doTrace
+			 proc flag)))))
 
 (define-syntax trace
   (syntax-rules ()
