@@ -1,6 +1,6 @@
 package kawa;
-import kawa.lang.CompileFile;
 import java.io.*;
+import java.net.*;
 import gnu.mapping.*;
 import gnu.expr.*;
 import gnu.text.SourceMessages;
@@ -455,7 +455,7 @@ public class repl extends Procedure0or1
 	      }
 	    try
 	      {
-		java.net.Socket socket = new java.net.Socket("localhost",port);
+		Socket socket = new Socket(InetAddress.getByName(null), port);
 		Telnet conn = new Telnet(socket, true);
 		java.io.InputStream sin = conn.getInputStream();
 		java.io.OutputStream sout = conn.getOutputStream();
@@ -666,7 +666,6 @@ public class repl extends Procedure0or1
               return iArg;
             String key = arg.substring(0, ci);
             String value = arg.substring(ci+1);
-            String uri, local, prefix = "";
             for (int i = 0; ; i++)
               {
                 String[] propertyField = propertyFields[i];
@@ -696,7 +695,7 @@ public class repl extends Procedure0or1
             // Run Language's static initializer.
             Language.getDefaultLanguage();
             Environment current = Environment.getCurrent();
-            current.put(symbol, null, value);
+            current.define(symbol, null, value);
           }
       }
     return something_done ? -1 : iArg;
