@@ -592,7 +592,6 @@ public class Scheme extends LispLanguage
       defProcStFld("setter", "gnu.kawa.functions.Setter", "setter");
       defSntxStFld("resource-uri", "kawa.lib.files");
 
-      defProcStFld("URI", "kawa.lib.files");
       defProcStFld("resolve-uri", "kawa.lib.files");
       defSntxStFld("module-uri", "kawa.lib.files");
 
@@ -752,6 +751,26 @@ public class Scheme extends LispLanguage
                     "gnu.text.PrettyWriter", "miserWidthLoc");
       defAliasStFld("html", "gnu.kawa.xml.XmlNamespace", "HTML");
 
+      defAliasStFld("path", "gnu.kawa.lispexpr.LangObjType", "pathType");
+      defAliasStFld("filepath", "gnu.kawa.lispexpr.LangObjType", "filepathType");
+      defAliasStFld("URI", "gnu.kawa.lispexpr.LangObjType", "URIType");
+      defProcStFld("path?", "kawa.lib.files");
+      defProcStFld("filepath?", "kawa.lib.files");
+      defProcStFld("URI?", "kawa.lib.files");
+      defProcStFld("absolute-path?", "kawa.lib.files");
+      defProcStFld("path-scheme", "kawa.lib.files");
+      defProcStFld("path-authority", "kawa.lib.files");
+      defProcStFld("path-user-info", "kawa.lib.files");
+      defProcStFld("path-host", "kawa.lib.files");
+      defProcStFld("path-port", "kawa.lib.files");
+      defProcStFld("path-file", "kawa.lib.files");
+      defProcStFld("path-parent", "kawa.lib.files");
+      defProcStFld("path-directory", "kawa.lib.files");
+      defProcStFld("path-last", "kawa.lib.files");
+      defProcStFld("path-extension", "kawa.lib.files");
+      defProcStFld("path-fragment", "kawa.lib.files");
+      defProcStFld("path-query", "kawa.lib.files");
+
       kawaEnvironment.setLocked();
   }
 
@@ -958,11 +977,6 @@ public class Scheme extends LispLanguage
         types.put ("f64vector", ClassType.make("gnu.lists.F64Vector"));
         types.put ("document", ClassType.make("gnu.lists.TreeList"));
         types.put ("readtable", ClassType.make("gnu.kawa.lispexpr.ReadTable"));
-        /* #ifdef use:java.net.URI */
-        types.put ("URI", ClassType.make("java.net.URI"));
-        /* #else */
-        // types.put ("URI", ClassType.make("java.lang.String"));
-        /* #endif */
       }
     Type type = (Type) types.get(name);
     if (type == null
@@ -990,6 +1004,12 @@ public class Scheme extends LispLanguage
       return getNamedType(name);
     if ("java.lang.String".equals(name))
       return Type.tostring_type;
+    if ("gnu.text.Path".equals(name))
+      return LangObjType.pathType;
+    if ("gnu.text.URIPath".equals(name))
+      return LangObjType.URIType;
+    if ("gnu.text.FilePath".equals(name))
+      return LangObjType.filepathType;
     return Type.make(clas);
   }
 
@@ -1047,7 +1067,9 @@ public class Scheme extends LispLanguage
     dispatchTable.set('\'', new ReaderQuote("syntax"));
     dispatchTable.set('`', new ReaderQuote("quasisyntax"));
     dispatchTable.set(',', ReaderDispatchMisc.getInstance());
-    tab.putReaderCtorFld("URI", "kawa.lib.files", "URI");
+    tab.putReaderCtorFld("path", "gnu.kawa.lispexpr.LangObjType", "pathType");
+    tab.putReaderCtorFld("filepath", "gnu.kawa.lispexpr.LangObjType", "filepathType");
+    tab.putReaderCtorFld("URI", "gnu.kawa.lispexpr.LangObjType", "URIType");
     tab.putReaderCtorFld("namespace", "kawa.lib.misc", "namespace");
     tab.putReaderCtorFld("duration", "kawa.lib.numbers", "duration");
     return tab;
