@@ -10,6 +10,11 @@ import gnu.bytecode.*;
 
 public class define_unit extends Syntax
 {
+  public static final define_unit define_unit = new define_unit(false);
+  static { define_unit.setName("define-unit"); }
+  public static final define_unit define_base_unit = new define_unit(true);
+  static { define_base_unit.setName("define-base-unit"); }
+
   /** True if this is define-base-unit, false if define-unit. */
   boolean base;
 
@@ -48,8 +53,8 @@ public class define_unit extends Syntax
 	      }
 	    if (unit != null)
 	      decl.noteValue(new QuoteExp(unit));
-	    p = tr.makePair(p, decl, p.cdr);
-	    st = tr.makePair(st, this, p);
+	    p = Translator.makePair(p, decl, p.cdr);
+	    st = Translator.makePair(st, this, p);
 	    forms.addElement (st);
 	    return true;
 	  }
@@ -106,9 +111,8 @@ public class define_unit extends Syntax
 							     args);
 	  }
       }
-    SetExp sexp = new SetExp (name, value);
+    SetExp sexp = new SetExp(decl, value);
     sexp.setDefining (true);
-    sexp.binding = decl;
     decl.noteValue(value);
     return sexp;
   }
