@@ -6,8 +6,11 @@ import gnu.lists.*;
 
 /** "define-variable" is like define, but ignored if variable already bound. */
 
-public class define_variable extends Syntax implements Printable
+public class define_variable extends Syntax
 {
+  public static final define_variable define_variable = new define_variable();
+  static { define_variable.setName("define-variable"); }
+
   public boolean scanForDefinitions (Pair st, java.util.Vector forms,
                                      ScopeExp defs, Translator tr)
   {
@@ -27,13 +30,14 @@ public class define_variable extends Syntax implements Printable
 	tr.push(decl);
 	decl.setSimple(false);
 	decl.setPrivate(true);
+	decl.setFlag(Declaration.IS_DYNAMIC);
 	if (mod.isStatic())
 	  decl.setFlag(Declaration.STATIC_SPECIFIED);
 	decl.setCanRead(true);
 	decl.setCanWrite(true);
 	decl.setIndirectBinding(true);
-	p = tr.makePair(p, decl, p.cdr);
-	st = tr.makePair(st, this, p);
+	p = Translator.makePair(p, decl, p.cdr);
+	st = Translator.makePair(st, this, p);
       }
     forms.addElement (st);
     return true;
@@ -72,7 +76,7 @@ public class define_variable extends Syntax implements Printable
     
     if (decl != null)
       {
-	sexp.binding = decl;
+	sexp.setBinding(decl);
 	if (decl.context instanceof ModuleExp
 	    && decl.getCanWrite())
 	  value = null;
