@@ -2,6 +2,7 @@ package kawa.standard;
 import kawa.lang.*;
 import gnu.lists.*;
 import gnu.mapping.*;
+import gnu.mapping.Location;  // As opposed to gnu.bytecode.Location.
 import gnu.expr.*;
 import gnu.bytecode.*;
 import gnu.kawa.reflect.Invoke;
@@ -11,9 +12,10 @@ import gnu.kawa.reflect.Invoke;
  * @author	Per Bothner
  */
 
-public class location extends Syntax implements Printable
+public class location extends Syntax
 {
-  static private Pattern pattern = new ListPat (2, 2);
+  public static final location location = new location();
+  static { location.setName("location"); }
 
   public Expression rewrite (Object obj, Translator tr)
   {
@@ -23,8 +25,8 @@ public class location extends Syntax implements Printable
     if (pair.cdr != LList.Empty)
       return tr.syntaxError ("extra arguments to location");
     //    Expression arg = tr.rewrite(pair.car);
-    Expression arg = tr.rewrite(pair.car);
-    return location.rewrite(arg, tr);
+    Expression[] args = { location.rewrite(tr.rewrite(pair.car), tr) };
+    return Invoke.makeInvokeStatic(thisType, "makeLocationProc", args);
   }
 
   private static ClassType thisType = ClassType.make("kawa.standard.location");
@@ -35,9 +37,15 @@ public class location extends Syntax implements Printable
       {
 	ReferenceExp rexp = (ReferenceExp) arg;
 	rexp.setDontDereference(true);
-	Declaration binding = rexp.getBinding();
-	if (binding != null && binding.isLexical())
-	  binding.setIndirectBinding(true);
+	Declaration decl = rexp.getBinding();
+	if (decl != null)
+          {
+            if (decl.isLexical())
+              decl.setIndirectBinding(true);
+            decl = Declaration.followAliases(decl);
+            decl.setCanRead(true);
+            decl.setCanWrite(true);
+          }
 	return rexp;
       }
     if (arg instanceof ApplyExp)
@@ -51,9 +59,29 @@ public class location extends Syntax implements Printable
     return tr.syntaxError("invalid argument to location");
   }
 
-  public static ProcLocation
+  public static Location
   makeProcLocation$V (Procedure proc, Object[] args)
   {
+    int nargs = args.length;
+    if (proc instanceof gnu.kawa.functions.ApplyToArgs
+        && nargs > 0
+        && args[0] instanceof Procedure) // FIXME
+      {
+        proc = (Procedure) args[0];
+        if (proc instanceof LocationProc && nargs == 1)
+          return ((LocationProc) proc).getLocation();
+        Object[] rargs = new Object[nargs-1];
+        System.arraycopy(args, 1, rargs, 0, rargs.length);
+        return new ProcLocation(proc, rargs);
+      }
+    if (proc instanceof LocationProc && nargs == 0)
+      return ((LocationProc) proc).getLocation();
     return new ProcLocation(proc, args);
   }
+
+  public static Procedure
+  makeLocationProc (Location loc)
+  {
+    return new LocationProc(loc);
+  }
 }
